#!/usr/bin/env python
#!/usr/bin/python
#
#
# check.py
# function library to check of the C3 configuration file
#
# Authors: G. Riccio
# Creation Date: October 15, 2015
# Release: 1.0
# Release Date: July 23, 2015
#
# Copyright 2016, Giuseppe Riccio, Massimo Brescia
# This file is part of C3.

    # C3 is free software: you can redistribute it and/or modify
    # it under the terms of the GNU General Public License as published by
    # the Free Software Foundation, either version 3 of the License, or
    # (at your option) any later version.

    # C3 is distributed in the hope that it will be useful,
    # but WITHOUT ANY WARRANTY; without even the implied warranty of
    # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    # GNU General Public License for more details.

    # You should have received a copy of the GNU General Public License
    # along with C3.  If not, see <http://www.gnu.org/licenses/>.
#
### WINDOWS INSTRUCTIONS
#
# This program has been produced with python 2.7.8 (64bit)
# It has been tested on Windows 7 and 10, by running it in a system
# shell window.
#
# IMPORTANT: be sure to have previously installed python
### LINUX UBUNTU INSTRUCTIONS
#
# This program has been tested on Linux Ubuntu 14.04 with python 2.7.6 (64-bit)
#
################
# SYNOPSIS
#
# Simple python source code to check of the C3 configuration file
#
###################

from    os	      import path,sep, makedirs
from 	libs.utility		import writeLogFile, getSelection, getJoinType, getCriteria, getErrorType, getAreaShape, getCoordSys, getUnits, setSeparator, getFormats, errorExit
import 	numpy	as np
import  shutil

def checkFile(filename, f, log):

	#check if file exists
	filename=filename.replace("\\",sep).replace("/",sep)

	if not path.isfile(filename): 							
		print("\nError: "+filename+" not found! Please retry")
		writeLogFile(log, "Error: "+filename+" not found\n", "a")		
		exit(1)
	
	#check type
	ftype=checkType(f, log)	

	return ftype
	

def checkType(f, log):				#ftype= 0 error, 1 csv, 2 ascii, 3 fits, htype= 0 noheader, 1 with header

	allowed=getFormats()
	if not f.lower() in allowed:	
		print("\nError: Format file "+f+" not recognized! Please retry")
		writeLogFile(log, "Format file "+f+" not recognized\n", "a")
		exit(1)

	else:

		ftype=f.lower();

	return ftype



def checkHeader(h):
	error=0
	htype=0
	if h.upper()=="Y":
		htype=1
	elif h.upper()=="N":
		htype=0
	else:
		error=1

	return htype, error

def checkStilts(stiltsDir, log):

	stiltsDir=stiltsDir.rstrip(sep)

	if not path.isfile(stiltsDir+sep+"stilts.jar"): 							#check stilts directory
		print("\nError: Stilts not found! Please retry")
		writeLogFile(log, "Error: Stilts not found\n", "a") 
		exit(1)

def checkTmp(tmp, log):
	tmp_path=tmp.rstrip(sep)

	#create tmp path
	if not path.exists(tmp_path):									#create tmp directory
		makedirs(tmp_path)
	else:
		print("\nWorking directory "+tmp_path+" already exists! Please delete it and retry.")
		writeLogFile(log, "Error: Working directory "+tmp_path+" already exists\n", "a") 
		exit(1)



def checkNumParams(dim_max, n_threads, dim_min, step_fwhm):
	if (dim_max<1) or (n_threads<0) or (dim_min<-1): 				#check if parameters values are right
		print("\nWrong parameters in configuration file! Please Retry")
		filelog=open("log.txt","a")
		filelog.write("Wrong parameters in configuration file\n")
		filelog.close()
		exit(1)
	if ((dim_max>1) or (dim_min>-1)) and (step_fwhm<=0) : 				
		print("\nWrong parameters in configuration file! Please Retry")
		filelog=open("log.txt","a")
		filelog.write("Wrong parameters in configuration file\n")
		filelog.close()
		exit(1)

def checkCoords(coordsys, units, log):
	checksystem(coordsys, log)
	units=[coordsys, units]
	checkunits(units,log)

def checksystem(coordsys, log):
	allowed=getCoordSys()
	if not coordsys in allowed:
		stringa=""
		for i in range(len(allowed)):
			stringa=stringa+"\""+allowed[i]+"\""
			if i!=len(allowed)-2:
				stringa=stringa+", "
			else:
				stringa=stringa+" or "
		stringa=stringa[:-2]

		print("\nCoordinate system error! Only "+stringa+" allowed. Please check and retry")
		writeLogFile(log, "Coordinate system error! Only "+stringa+" allowed.\n", "a") 		
		exit(1)
	
def checkunits(units, log):
	coordsys=units[0].upper().strip()
	coordunits=units[1].lower().strip()
	allowed=getUnits()
	

	if not coordunits in allowed:
		stringa=""
		for i in range(0,len(allowed)/2):
			stringa=stringa+"\""+allowed[i]+"\""+", "
		stringa=stringa[:-2]

		print("Coordinate units error! Only "+stringa+" and their abbreviations allowed. Please check and retry")
		writeLogFile(log, "Coordinate units error! Only "+stringa+" and their abbreviations allowed allowed\n", "a") 		
		exit(1)
	

def checkCriteria(criteria, logfile):

	critarray=getCriteria()
	c=len(np.where(critarray==criteria)[0])
	if c==0:
		print("\nError! Match criteria algorithm not valid. Please check and retry")
		writeLogFile(logfile, "Error! Match criteria algorithm not valid\n", "a")
		exit(1)



def checkSelection(selection, join, logfile):

	selarray=getSelection()
	joinarray=getJoinType()

	s=len(np.where(selarray==selection)[0])
	j=len(np.where(joinarray==join)[0])
	
	if s==0:
		print("\nError! Match selection parameter: "+selection.upper()+" not allowed. Please check and retry")
		writeLogFile(logfile, "Error! Match selection parameter: "+selection.upper()+" not allowed\n", "a")
		exit(1)

	if j==0:
		print("\nError! Join type parameter: "+join.upper()+" not allowed. Please check and retry")
		writeLogFile(logfile, "Error! Join type parameter: "+join.upper()+" not allowed\n", "a")
		exit(1)

	
def checkSkyPars(error, shape, logfile):
				
	errarray=getErrorType()
	shapearray=getAreaShape()

	e=len(np.where(errarray==error)[0])
	s=len(np.where(shapearray==shape)[0])
	
	if e==0:
		print("\nError! Error type parameter: "+error.upper()+" not allowed. Please check and retry")
		writeLogFile(logfile, "Error! Error type parameter: "+error.upper()+" not allowed\n", "a")
		exit(1)

	if s==0:
		print("\nError! Area shape parameter: "+shape.upper()+" not allowed. Please check and retry")
		writeLogFile(logfile, "Error! Area shape parameter: "+shape.upper()+" not allowed\n", "a")
		exit(1)



def checkCatSize(rows1,rows2,logfile, tmp_path):
	if len(rows1)==0:
		print("\nCatalogue 1 has no rows. Exit!")
		writeLogFile(logfile, "\nCatalogue 1 has no rows. Exit!\n\n", "a")
		shutil.rmtree(tmp_path)
		exit(1)
	if len(rows2)==0:
		print("\nCatalogue 2 has no rows. Exit!")
		writeLogFile(logfile, "\nCatalogue 2 has no rows. Exit!\n\n", "a")
		shutil.rmtree(tmp_path)
		exit(1)

