#!/usr/bin/env python
#!/usr/bin/python
#
##################### ViaLactea DATA MINING TOOLS
#
# ellipseCalc.py
# function library to calculate the ellipse of a reference source object,
# given its parameters, and to verify if a new object is inside it
#
# Authors: M. Brescia, S. Cavuoti
# Creation Date: May 21, 2014
# Release: 6.0
# Release Date: October 24, 2014
#
# Copyright 2016, Giuseppe Riccio, Massimo Brescia
# This file is part of C3.

    # C3 is free software: you can redistribute it and/or modify
    # it under the terms of the GNU General Public License as published by
    # the Free Software Foundation, either version 3 of the License, or
    # (at your option) any later version.

    # C3 is distributed in the hope that it will be useful,
    # but WITHOUT ANY WARRANTY; without even the implied warranty of
    # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    # GNU General Public License for more details.

    # You should have received a copy of the GNU General Public License
    # along with C3.  If not, see <http://www.gnu.org/licenses/>.
#
#################### WINDOWS INSTRUCTIONS
#
# This program has been produced with python 2.7.8 (64bit)
# It has been tested on Windows 8pro, by running it in a system
# shell window.
#
# IMPORTANT: be sure to have previously installed python 2.7.8
# 64-bit version, together with the 
# following external packages:
#
# 	astropy;
# 	matplotlib;
# 	numpy;
# 	pillow;
# 	pyfits;
# 	pyparsing;
# 	dateutil;
# 	pandas;
# 	scipy
#
### LINUX UBUNTU INSTRUCTIONS
#
# This program has been tested on Linux Ubuntu 12.04 with python 2.7.5 (64-bit)
# Under ubuntu the above listed libraries can be installed in the following way:
#
# >> sudo apt-get install python-numpy* python-scipy* python-pyfits python-pyparsing python-dateutil  python-matplotlib*
#
#############################################################################

# import needed libraries
from math	import pi, cos, sin, sqrt
from libs.utility 	import correctedDistance
constant = 206264.806247

#++++++++++++++++++++ subroutine 
# conversion from arcsec to radiants
# rad = arcsec / constant
def asec2rad(x):
	a2r = x / constant
	return a2r

#++++++++++++++++++++ subroutine 
# conversion from degrees to radiants
# rad = deg * pi / 180.0
def deg2rad(x):
	d2r = x * pi / 180.0
	return d2r

#++++++++++++++++++++ subroutine 
# conversion from arcsec to radiants
# arcsec = rad * constant
def rad2asec(x):
	r2a = x * constant
	return r2a

#++++++++++++++++++++ subroutine 
# conversion from arcsec to deg
# deg = (arcsec / 3600.0)
def asec2deg(x):
	a2d = x / 3600.0
	return a2d

#++++++++++++++++++++ subroutine 
# conversion from deg to arcsec
# arcsec = (deg * 3600.0)
def deg2asec(x):
	d2a = x * 3600.0
	return d2a


#++++++++++++++++++++ subroutine 
# perform a roto-translation of coordinates
def rotoTranslation(glon, glat, glonc, glatc, phi):

	# calculate translation coords of new obj
	X1 = correctedDistance(glon , glonc, "glon") * cos((asec2deg(glat)+asec2deg(glatc))/360*pi)
	Y1 = correctedDistance(glat , glatc, "glat")

	#X1=glon-glonc
	#Y1=glat-glatc

	# calculate rotation coords of new obj
	X2 = (X1 * cos(phi)) - (Y1 * sin(phi))
	Y2 = (X1 * sin(phi)) + (Y1 * cos(phi))

	return X2, Y2


#++++++++++++++++++++ subroutine 
# calculate ellipse on x,y
# axa, axb are ellipse axes
def ellipse(x, y, axa, axb):

	# find major anf minor semi-axes
	if axa > axb:
		a = axa / 2.0
		b = axb / 2.0
	else:
		b = axa / 2.0
		a = axb / 2.0
    
	# calculate dist as ellipse equation with new obj
	ell = ((x*x) / (a*a)) + ((y*y) / (b*b))
	dist=sqrt((x*x)+(y*y))
	return ell,dist

#++++++++++++++++++++ subroutine 
# verify object in/out of the ellipse
def isInside(dist):
	if dist > 1:
		ans = 0
	else:
		ans = 1

	return ans

#++++++++++++++++++++ subroutine 
# MAIN subroutine CODE
# this is the routine to be externally called
def matchVerify(glon, glat, glonc, glatc, FWHMA, FWHMB, PA, shape):
	
	# conversion into arcsec
	glon   = deg2asec(glon)
	glat  = deg2asec(glat)
	glonc  = deg2asec(glonc)
	glatc = deg2asec(glatc)
	
##### PATCH HIGAL FORMAT (PA referred always to FWHMA)
	if FWHMB>FWHMA:
		PA=PA+90
##### END OF THE PATCH
	phi  = deg2rad(PA)
	x,y = rotoTranslation(glon, glat, glonc, glatc, phi)	#x and y are in arcsecs

	if shape=="ellipse":
		ell, d = ellipse(x, y, FWHMA, FWHMB)
		fin_ans = isInside(ell)
	elif shape=="rectangle":
		d, fin_ans = rectangle(x, y, FWHMA, FWHMB)

	return fin_ans, d

def rectangle(x, y, axa, axb):

    # find major anf minor semi-axes
	if axa > axb:
		a = axa / 2.0
		b = axb / 2.0
	else:
		b = axa / 2.0
		a = axb / 2.0


	#print abs(x)< a, abs(y)< b
	if abs(x)<a and abs(y)<b:
		fin_ans=1
	else:
		fin_ans=0 
	
	d=sqrt(x*x+y*y)

	return d, fin_ans	

