#!/usr/bin/env python
#!/usr/bin/python
#
# ioUtils.py
# function library to provide utility to other I/O functions
#
# Authors: G. Riccio
# Creation Date: March 1, 2016
# Release: 1.0
# Release Date: July 23, 2016
#
# Copyright 2016, Giuseppe Riccio, Massimo Brescia
# This file is part of C3.

    # C3 is free software: you can redistribute it and/or modify
    # it under the terms of the GNU General Public License as published by
    # the Free Software Foundation, either version 3 of the License, or
    # (at your option) any later version.

    # C3 is distributed in the hope that it will be useful,
    # but WITHOUT ANY WARRANTY; without even the implied warranty of
    # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    # GNU General Public License for more details.

    # You should have received a copy of the GNU General Public License
    # along with C3.  If not, see <http://www.gnu.org/licenses/>.
#
### WINDOWS INSTRUCTIONS
#
# This program has been produced with python 2.7.8 (64bit)
# It has been tested on Windows 7 and 8pro, by running it in a system
# shell window.
#
# IMPORTANT: be sure to have previously installed python 2.7.8
# 64-bit version together with the following external packages:
#
# 	pyfits;
#
### LINUX UBUNTU INSTRUCTIONS
#
# This program has been tested on Linux Ubuntu 14.04 with python 2.7.6 (64-bit)
#
#################
# SYNOPSIS
#
# Simple python source code implementing utility functions 
#
###################
#from 	string		import split, replace
import  configparser
from 	os 		import sep, system, path, makedirs
from    math	    import floor,ceil
import  csv
import  re
from 	libs.check 		import checkFile, checkCriteria, checkSelection, checkSkyPars, checkCoords, checkStilts, checkTmp
from 	libs.utility		import setSeparator, writeLogFile, file2csv, findCol, errorExit
import	pyfits
from 	multiprocessing 	import cpu_count
from sys import getsizeof

def getConf(configfile):
	global 	conf 
	conf= configparser.ConfigParser(inline_comment_prefixes=('#',';'))			# tries to read the configuration file
	conf.read(configfile)


def ConfigSectionMap(section):					# reads a parameter from the configuration file
	dict1 = {}
	options = Config.options(section)
	for option in options:
		try:
			dict1[option] = Config.get(section, option)
			if dict1[option] == -1:
				DebugPrint("skip: %s" % option)
		except:
			print("exception on %s!" % option)
			dict1[option] = None
	return dict1   
	

def getLogfile(fileConf):					# return the name of the log file
	getConf(fileConf)
	global ConfLog
	ConfLog = configparser.ConfigParser(inline_comment_prefixes=('#',';'))		    # tries to read the configuration file
	ConfLog.read(fileConf)

	try:	
		logname=conf.get("I/O Files","log file")
		if logname!="":
			logname=setSeparator(logname)	
			logpezzi=logname.split(sep)
			logdir=logname[:-len(logpezzi[-1])]
			if logdir=='':
				logdir="."+sep
			if not path.exists(logdir):
				makedirs(logdir)
			print("\nLog File: "+logdir+logpezzi[-1])
			
		else:
			logname="."+sep+"log.txt"
			print("\nLog File Name not set, using "+logname)
	
	except:
			logname="."+sep+"log.txt"
			print("\nLog File Name not set, using "+logname)
	
	return logname

def getDirs(configfile, logfile):

#	conf = configparser.ConfigParser()			# tries to read the configuration file
#	conf.read(configfile)
#	
	try:
		stiltsDir=		setSeparator(conf.get("I/O Files","stilts directory"))
		tmp_path=		setSeparator(conf.get("I/O Files","working directory"))


	except:
		print("\nError reading configuration file! Please check and retry")
		writeLogFile(logfile, "Error reading configuration file\n", "a")
		exit(1)
	checkStilts(stiltsDir, logfile)
	stiltsDir=stiltsDir+sep
	checkTmp(tmp_path, logfile)

	return stiltsDir, tmp_path



	
def setOutDir(filename):
	filename=setSeparator(filename)
	fileparts=filename.split(sep)
	outfile=fileparts[-1]
	filedir=filename[:-len(fileparts[-1])]
	if filedir=='':
		filedir="."+sep
	if not path.exists(filedir):
		makedirs(filedir)	

	return filedir, outfile



def readFile(filename, f, log, stiltsDir, tmp_path):
	

	filetmp=filename
	if f!="csv":
		filetmp=file2csv(filename, f, stiltsDir, tmp_path, log)

	else:

		f1=open(filename,"r")
		hasH = csv.Sniffer().has_header(f1.read(1000000))
		f1.seek(0)
		if not hasH:
			# read csv
			reader=csv.reader(f1)
			data=[line for line in reader]
			
			ncol=len(data[0])
			header=""		
			#create header
			for i in range(1,ncol+1):
				header+="col"+str(i)+","

			header=header[:-1]
			# create tmp file
			csvtmp=open(tmp_path+sep+"tmpcsv.csv", "wb")
			csvtmp.write(header+"\n")
			wr = csv.writer(csvtmp, delimiter=",")
			wr.writerows(data)
			
			csvtmp.close()
			filetmp=tmp_path+sep+"tmpcsv.csv"
		f1.close()

	try:
		f2   = open(filetmp,"r")
		# read the header
		head2 = f2.readline()
		header2 = head2.replace("\r", "").replace("\n","")
		# read all rows but skip the header
		rows2=f2.readlines()
		f2.close()
	except:
		print("\nError opening input file: "+ filename+ "! Please check and retry")
		writeLogFile(log, "Error opening input file "+filename+"\n", "a")		
		exit(1)
	
	return filetmp, header2, rows2


def readCriteria(configfile, logfile):
	
	criteria=""
#	conf = configparser.ConfigParser()			# tries to read the configuration file
#	conf.read(configfile)
	
	try:
		criteria=		conf.get("Match Criteria","algorithm")
		criteria=criteria.lower()

	except:
		print("\nError reading configuration file! Please check and retry")
		writeLogFile(logfile, "Error reading configuration file\n", "a")
		exit(1)

	checkCriteria(criteria, logfile)

	return criteria

def readSelection(configfile, logfile):
	
	selection=""
	join=""

#	conf = configparser.ConfigParser()			# tries to read the configuration file
#	conf.read(configfile)
	
	try:
		selection=		conf.get("Output Rows","match selection").lower()
		join=			conf.get("Output Rows","join type").lower()
	except:
		print("\nError reading output configuration parameters! Please check and retry")
		writeLogFile(logfile, "Error reading output configuration parameters\n", "a")
		exit(1)

	checkSelection(selection, join, logfile)	
	
	return selection, join



def getIO(configfile, logfile):
	
#	io=configparser.ConfigParser()			# tries to read the configuration file
#	io.read(configfile)
	
	try:
		file1=					conf.get("I/O Files","input catalogue 1")
		file2=					conf.get("I/O Files","input catalogue 2")
		outputFile=				conf.get("I/O Files","output")
		format1=				conf.get("I/O Files","format catalogue 1")
		format2=				conf.get("I/O Files","format catalogue 2")
		outformat=				conf.get("I/O Files","output format")
	except:
		print("\nError reading configuration file! Please check and retry")
		writeLogFile(logfile, "Error reading configuration file\n", "a")
		exit(1)	
	

	return file1, file2, outputFile, format1, format2, outformat



def readMatchCol(h1,h2, configfile, logfile,tmp_path):

#	cols=configparser.ConfigParser()			# tries to read the configuration file
#	cols.read(configfile)

	try:
		match1=				conf.get("Exact Value parameters","input 1 column")
		match2=				conf.get("Exact Value parameters","input 2 column")
	except:
		print("\nError reading configuration file! Please Retry")
		writeLogFile(logfile, "Error reading configuration file\n", "a")
		exit(1)	
	
	matchcol1=findCol(h1, match1, logfile,tmp_path)-1
	matchcol2=findCol(h2, match2, logfile,tmp_path)-1


	return matchcol1, matchcol2

def getDesignationColumn(h1, h2, configfile, logfile,tmp_path):
	

	try:
		d1=				conf.get("Catalogue 1 Properties","designation column")

	except:
		d1=-1

	if d1!=-1:
		des1=findCol(h1, d1, logfile,tmp_path)-1
	else:
		des1=d1
	

	try:
		d2=				conf.get("Catalogue 2 Properties","designation column")

	except:
		d2=-1

	if d2!=-1:
		des2=findCol(h2, d2, logfile,tmp_path)-1
	else:
		des2=d2


	
	
	return des1, des2


def getNThreads(configfile, logfile, tmp_path):
	
#	nth=configparser.ConfigParser()			# tries to read the configuration file
#	nth.read(configfile)
	n_threads=cpu_count()-1
	error=0
	try:
		n=				conf.get("Threads Properties","thread limit")

	except:
		n="default"


	try:
		n_threads=int(n)
		if n_threads<=0:
			print("\nError! Number of concurrent threads less or equal to 0. Please check and retry")
			writeLogFile(logfile, "Error! Number of concurrent threads less or equal to 0\n", "a")
			error=1
	except:	
		if n=="default":
			print("\nUsing default Thread limit value (CPUcount-1)")
			
		else:
			print("\nError reading Thread limit value! Please check and retry")
			writeLogFile(logfile, "Error reading Thread limit value\n", "a")
			error=1
	if error==1:
		errorExit(tmp_path)
	
	return n_threads


def readSkyType(configfile, logfile):

	error=""
	shape=""

#	conf = configparser.ConfigParser()			# tries to read the configuration file
#	conf.read(configfile)
	
	try:
		error=			conf.get("Sky parameters","size type").lower()
		shape=			conf.get("Sky parameters","area shape").lower()
	except:
		print(error+","+shape)
		print("\nError reading Sky parameters! Please check and retry")
		writeLogFile(logfile, "Error reading Sky parameters\n", "a")
		exit(1)

	checkSkyPars(error, shape, logfile)	
	
	return error, shape
	

def readFixedValues(configfile, logfile):
	
#	conf = configparser.ConfigParser()			# tries to read the configuration file
#	conf.read(configfile)
	
	try:
		axis1=			float(conf.get("Sky parameters","matching area first dimension").lower())	#it is the radius of the ellipse	
		axis2=			float(conf.get("Sky parameters","matching area second dimension").lower())

	except:
		print("\nError reading Sky parameters! Please check and retry")
		writeLogFile(logfile, "Error reading Sky parameters\n", "a")
		exit(1)

	if axis1<=0 or axis2<=0:
		print("\nError! Area dimensions less or equal to 0. Please check and retry")
		writeLogFile(logfile, "Error! Area dimensions less or equal to 0.\n", "a")
		exit(1)			
	
	return axis1, axis2


def readFWHMValues(h1, configfile, logfile,tmp_path):
#	conf = configparser.ConfigParser()			# tries to read the configuration file
#	conf.read(configfile)

	try:
		fwhm1_col=			conf.get("Sky parameters","matching area first dimension")
		fwhm2_col=			conf.get("Sky parameters","matching area second dimension")
		factor=				float(conf.get("Sky parameters","parametric factor"))
	except:
		print("\nError reading configuration file! Please Retry")
		writeLogFile(logfile, "Error reading configuration file\n", "a")
		exit(1)	

	fwhm1=findCol(h1, fwhm1_col, logfile,tmp_path)-1
	fwhm2=findCol(h1, fwhm2_col, logfile,tmp_path)-1

	return fwhm1, fwhm2, factor


def readCoordsPars(configfile, logfile, n, h,tmp_path):

#	conf = configparser.ConfigParser()			# tries to read the configuration file
#	conf.read(configfile)

	try:
		coordsys_name=			conf.get("Catalogue "+str(n)+" Properties","coordinate system")

		#glon_s=				conf.get("Catalogue "+str(n)+" Properties",'glon/ra column')
		#glat_s=				conf.get("Catalogue "+str(n)+" Properties",'glat/dec column')

		#glon=glon_s.split(",")
		#glat=glat_s.split(",")
		#glon_name=glon[0]
		#glat_name=glat[0]
		#glon_unit=glon[1].strip()
		#glat_unit=glat[1].strip()

		units=				conf.get("Catalogue "+str(n)+" Properties",'coordinate units')
		#glat=unit=			conf.get("Catalogue "+str(n)+" Properties",'coordinate units')
		glon_name=			conf.get("Catalogue "+str(n)+" Properties",'glon/ra column')
		glat_name=			conf.get("Catalogue "+str(n)+" Properties",'glat/dec column')


	except:
		print("\nError reading configuration file! Please Retry")
		writeLogFile(logfile, "Error reading configuration file\n", "a")
		errorExit(tmp_path)	

	#checkCoords(coordsys_name, glon_unit, glat_unit, logfile)
	checkCoords(coordsys_name, units, logfile)	
	glon_col=findCol(h, glon_name, logfile,tmp_path)-1
	glat_col=findCol(h, glat_name, logfile,tmp_path)-1

#	return coordsys_name, glon_col, glat_col, glon_unit, glat_unit
	return coordsys_name, glon_col, glat_col, units


def readPA(configfile, logfile, h, errorType,tmp_path):

	pa_name="default"
	pa_col="default"
	try:
		pa_name=			conf.get("Sky parameters","pa column/value")

	except:
		pa_col="default"
	
	
	if pa_name!="default":
		if errorType=="parametric":
			pa_col=findCol(h, pa_name,logfile,tmp_path)-1
		else:
			try:
				pa_col=float(pa_name)
			except:
				print("\nError reading PA value! Please check and retry\n")
				writeLogFile(logfile, "\nError reading PA value\n", "a")
				errorExit(tmp_path)

	try:
		pa_settings=			conf.get("Sky parameters","pa settings")
		if pa_settings!="default":
			clock_name=pa_settings.split(",")[0]
			shift=float(pa_settings.split(",")[1])
		else:
			clock_name="clock"
			shift=0.0
	except:
		clock_name="clock"
		shift=0.0

	if clock_name=="clock":
		clock=1
	elif clock_name=="counter":
		clock=-1
	else:
		print("Error reading PA settings! Please check and retry.")
		writeLogFile(logfile, "Error reading PA settings\n", "a")
		errorExit(tmp_path)	

	return pa_col, clock, shift

 
def concatOutput(tmp_path, stiltsDir, outputFile, outformat, n, logfile):
#	infile=""
	out=tmp_path+sep+"final.csv"
	outputFile=outputFile.replace("/",sep).replace("\\", sep)
	i=0
	n_matches=-1

	while i<n:
		temp_file=tmp_path+sep+"final_"+str(i)+".csv"

		if path.isfile(temp_file):
			ftmp=open(temp_file,"r")
			if path.isfile(out):
				ftmp.readline()			#skip header
			
			fout=open(out,"a")
			line=ftmp.readline()
			while line!="":
				fout.write(line)

				line=ftmp.readline()
				n_matches+=1

			fout.close()
			
		i+=1


	if n_matches>0:
		cmd="java -jar "+stiltsDir+"stilts.jar tcopy "+out+" "+outputFile+" ifmt=csv ofmt="+outformat 
		system(cmd)
	else:
		with open(outputFile+".txt","w") as f:
			f.write("No matches found.")		

	return n_matches

	#	if path.isfile(temp_file):
	#		infile=infile+" "+temp_file
	#	i+=1
	#cmd="java -jar "+stiltsDir+"stilts.jar tcatn nin="+str(n)+" ifmt=csv in='"+infile.strip()+"' out="+outputFile+" ofmt="+outformat
	#system(cmd)


def getCellSize(configfile, logfile, tmp_path):
	
	cellsize=100
	error=0
	try:
		c=				conf.get("Sky parameters","Catalogue 2 minimum partition cell size")

	except:
		c="default"


	try:
		cellsize=int(c)
		if cellsize<=0:
			print("\nError! Minimum size of partition cells less or equal to 0. Please check and retry")
			writeLogFile(logfile, "Error! Minimum size of partition cells less or equal to 0\n", "a")
			error=1
	except:	
		if c=="default":
			print("\nUsing default Minimum partition cell size\n")
			
		else:
			print("\nError reading Minimum partition cell size value! Please check and retry")
			writeLogFile(logfile, "Error reading Minimum particion cell size value\n", "a")
			error=1
	if error==1:
		errorExit(tmp_path)
	
	return cellsize






