#!/usr/bin/env python
#!/usr/bin/python
#
# prePhase.py
# function library to provide utilities for preprocessing phase
#
# Authors: G. Riccio
# Creation Date: March 1, 2016
# Release: 1.0
# Release Date: July 23, 2016
#
# Copyright 2016, Giuseppe Riccio, Massimo Brescia
# This file is part of C3.

    # C3 is free software: you can redistribute it and/or modify
    # it under the terms of the GNU General Public License as published by
    # the Free Software Foundation, either version 3 of the License, or
    # (at your option) any later version.

    # C3 is distributed in the hope that it will be useful,
    # but WITHOUT ANY WARRANTY; without even the implied warranty of
    # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    # GNU General Public License for more details.

    # You should have received a copy of the GNU General Public License
    # along with C3.  If not, see <http://www.gnu.org/licenses/>.
#
### WINDOWS INSTRUCTIONS
#
# This program has been produced with python 2.7.8 (64bit)
# It has been tested on Windows 7 and 8pro, by running it in a system
# shell window.
#
# IMPORTANT: be sure to have previously installed python 2.7.8
# 64-bit version
#
### LINUX UBUNTU INSTRUCTIONS
#
# This program has been tested on Linux Ubuntu 14.04 with python 2.7.6 (64-bit)
#
#################
# SYNOPSIS
#
# Simple python source code implementing utility functions for the preprocessing phase of the algorithm "sky"
#
###################

from 	os 			import sep, system, path
import  csv
import  re
from 	libs.utility 		import writeLogFile,errorExit
import 	sys
from 	multiprocessing 	import Process

def newRowsCat1(rows, filename, coordsys, units, id_col, coord1_col, coord2_col, axis1, axis2, errorType, factor, pa_col, tmp_path, header,stiltsDir, logfile, n_threads):			#create array containing only useful data for matching from catalogue 1	

	header_cols=header.split(",")
	
	c1=header_cols[coord1_col]
	c2=header_cols[coord2_col]

	convertCoords(filename, coordsys, units, 1, tmp_path, c1, c2, stiltsDir,logfile)	#create 2 files with converted coordinates


	fcoord=open(tmp_path+sep+"coord_cat"+str(1)+".csv", "r")	#open catalogue 1 converted coordinates file
	headcoord=fcoord.readline()					#header
	#lines=fcoord.readlines()					
	counts=len(rows)/10						# for progress bar
	n=1								# for progress bar
	newrows=[]

	groupID=0
	f=open(tmp_path+sep+"group"+str(groupID),"w")

	for i in range(len(rows)):					# for each line of the input rows array
	
		r=rows[i].rstrip("\n").split(",")
		if id_col>=0:				#get ID 
			C_ID=r[id_col]
		else:					#or assign an incremental index as ID (if missing)
			C_ID=i+1

		coord1=r[coord1_col]
		coord2=r[coord2_col]
		
		#coords=lines[i].split(",")		#split the corresponding i-th line of the new coordinates file
		coords=fcoord.readline().split(",")		
		l=coords[0]

		b=coords[1].rstrip("\n")
		
		if errorType=="parametric":			# if fwhm, axis1 and axis2 are the column indexes of the fwhms
			fwhm1=r[axis1]
			fwhm2=r[axis2]
			if fwhm1=="":
				fwhm1='0.0'

			if fwhm2=="":
				fwhm2='0.0'


		else:					# if fixed, axis1 and axis2 are the values of the ellipse/rectangle
			fwhm1=axis1
			fwhm2=axis2
			
		if pa_col=="default":		# if pa_col is default = 0 degrees
			pa=0		
		elif errorType=="parametric":
			pa=r[pa_col]		
		else:
			pa=pa_col

		if pa=="":
			pa='0.0'


		#newstring=[C_ID,coord1,coord2,l,b,factor*float(fwhm1),factor*float(fwhm2),float(pa),i,rows[i].rstrip("\n").replace(",","_")]
		
		newstring=str(C_ID)+","+str(coord1)+","+str(coord2)+","+str(l)+","+str(b)+","+str(factor*float(fwhm1))+","+str(factor*float(fwhm2))+","+str(float(pa))+","+str(i)+","+rows[i].rstrip("\n").replace(",","_")+"\n"
		
		#print(str(i)+","+str(groupID))

		if i<=(groupID+1)*(len(rows)/n_threads):
			f.write(newstring)
		else:
			if groupID<n_threads-1:
				groupID+=1
				f.close()
				f=open(tmp_path+sep+"group"+str(groupID),"w")
			f.write(newstring)

	



	#	rows[i]=newstring

		#newrows.append(newstring)






	#	if i==counts*n:				# progress bar
	#		sys.stdout.write(".")
	#		sys.stdout.flush()
	#		n+=1
	f.close()
	sys.stdout.write("..........")
	sys.stdout.flush()
	#return rows


def convertCoords(filename, coordsys, units, catnum, tmp_path, c1, c2, stiltsDir,logfile):	#coordinate converter from (a,b) in (glon,glat)

	try:
		if sep=="/":
			cmd="java -jar "+stiltsDir+sep+"stilts.jar tpipe ifmt=csv in="+filename+" cmd='addskycoords -inunit "+units+" "+coordsys+" gal "+c1+" "+c2+" coord1 coord2' cmd='keepcols \"coord1 coord2\"' omode=out out="+tmp_path+sep+"coord_cat"+str(catnum)+".csv ofmt=csv"
		elif sep=="\\":
			cmd="java -jar "+stiltsDir+sep+"stilts.jar tpipe ifmt=csv in="+filename+" cmd=\"addskycoords -inunit "+units+" "+coordsys+" gal "+c1+" "+c2+" coord1 coord2\" cmd=\"keepcols 'coord1 coord2'\" omode=out out="+tmp_path+sep+"coord_cat"+str(catnum)+".csv ofmt=csv"
		system(cmd)

	except:
		print("Error converting coordinates! Please check and retry")
		writeLogFile(log, "Coordinate system error! Only "+stringa+" allowed.\n", "a") 		
		errorExit(tmp_path)

def createSubDiv(n_parts, xmin, xmax, step):	#divide region [lmin-lmax] into sub-region depending on step
	subdiv={}
	for i in range(1,n_parts+1):
		subdiv[i]=i*step+xmin
	#subdiv[n_parts]=xmax+1
	
	return subdiv	

def splitRows(newrows, subL, subB, header, tmp_path):		#split catalogue 2 rows in the sub-regions previously defined

	for r in range(len(newrows)):
		#stringa=newrows[r][0]
		l=newrows[r][1]
		b=newrows[r][2]
		#stringa=newrows[r][3].replace(",","_")
		stringa=newrows[r][0].replace(",","_")
		
		if len(subL)==1:				# if there is 1 element, it is in part 1
			partl=1
		else:
			for lp in range(1,len(subL)+1):
				lmax=subL[lp]			#max l of the subregion
				if lp!=1:			#min l of the subregion (the max of the previous one)
					lmin=subL[lp-1]
				else:
					lmin=-999
				if l>=lmin and l<lmax:		#check if l belongs to the interval [lmin, lmax[
					partl=lp
					break


		if len(subB)==1:
			partb=1
		else:
			for bp in range(1,len(subB)+1):
				bmax=subB[bp]
				if bp!=1:
					bmin=subB[bp-1]
				else:
					bmin=-999

				if b>=bmin and b<bmax:
					partb=bp
					break


		with open(tmp_path+sep+"part_"+str(partl)+"_"+str(partb)+".csv","a") as f:
			#f.write(str(r)+","+stringa+","+str(l)+","+str(b)+"\n")
			f.write(str(r)+","+str(l)+","+str(b)+","+stringa+"\n")



def preExact(rows1,rows2, matchcol1, matchcol2, n_threads, tmp_path):

	proc1=Process(target=preExact1, args=(rows1,matchcol1,n_threads, tmp_path,))
	proc2=Process(target=preExact2, args=(rows2, matchcol2, tmp_path,))
	proc1.start()
	proc2.start()
	proc1.join()
	proc2.join()
			



def preExact1(rows, matchcol, n_threads, tmp_path):
	
	groupID=0
	f=open(tmp_path+sep+"group"+str(groupID),"w")
	step=int(len(rows)/n_threads)
	for i in range(0,len(rows)):
		val=rows[i].split(",")[matchcol]
		if i<=(groupID+1)*(int(len(rows)/n_threads)):
			f.write(str(i)+","+val+"\n")
		else:
			if groupID<n_threads-1:
				groupID+=1
				f.close()
				f=open(tmp_path+sep+"group"+str(groupID),"w")
			f.write(str(i)+","+val+"\n")
		
	f.close()

def preExact2(rows, matchcol, tmp_path):

	f=open(tmp_path+sep+"cat2exact.csv","w")
	for r in rows:
		val=r.split(",")[matchcol]
		f.write(val+"\n")


	f.close()
		





