#!/usr/bin/env python

#
#
# threadSky.py
# function library to create threads to perform the crossmatch
#
# Authors: G. Riccio
# Creation Date: March 5, 2016
# Release: 1.0
# Release Date: July 23, 2016
#
# Copyright 2016, Giuseppe Riccio, Massimo Brescia
# This file is part of C3.

    # C3 is free software: you can redistribute it and/or modify
    # it under the terms of the GNU General Public License as published by
    # the Free Software Foundation, either version 3 of the License, or
    # (at your option) any later version.

    # C3 is distributed in the hope that it will be useful,
    # but WITHOUT ANY WARRANTY; without even the implied warranty of
    # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    # GNU General Public License for more details.

    # You should have received a copy of the GNU General Public License
    # along with C3.  If not, see <http://www.gnu.org/licenses/>.
#
### WINDOWS INSTRUCTIONS
#
# This program has been produced with python 2.7.8 (64bit)
# It has been tested on Windows 7 and 10, by running it in a system
# shell window.
#
# IMPORTANT: be sure to have previously installed python 2.7.8
# 64-bit version, together with the following external packages:
#
# 	numpy;
#	pyfits;
#
### LINUX UBUNTU INSTRUCTIONS
#
# This program has been tested on Linux Ubuntu 14.04 with python 2.7.6 (64-bit)
#
#################
# SYNOPSIS
#
# Simple python source code that initialize threads to perform
# the match required to associate two objects from two catalogues 
#
###################


# import needed libraries

import  threading
import  time
from    os              import sep, system, path
from 	libs.distCalc	import matchVerify
from    libs.utility    import assignPart
from    math            import sqrt, floor, ceil
import	csv
import  numpy as np
import 	sys
from 	multiprocessing import Process, Queue, cpu_count
import 	warnings
import 	gc
import ast


def set_globvar(area, tmp, n, mdir):
	global lmin
	global lmax
	global bmin
	global bmax
	global subdiv_l
	global n_parts_l
	global subdiv_b
	global n_parts_b
	global shape
	global clock
	global shift	
	global tmp_path
	global maindir
	global n_threads




	lmin=float(area[0])
	lmax=float(area[1])
	bmin=float(area[2])
	bmax=float(area[3])
	n_parts_l=float(area[4])
	n_parts_b=float(area[5])
	subdiv_l=area[6]
	subdiv_b=area[7]
	shape=area[8]
	clock=area[9]
	shift=area[10]
	maindir=mdir
	tmp_path=tmp
	n_threads=n


       
def matchSky(C_ID,glonc,glatc,PA,fwhm1,fwhm2,parts_string,tmp_path,ind,shape,stringa_c):
	rows=[]
	res=""
	parts=parts_string.split(",")
	ok=0
	for part in parts:
		partfile=tmp_path+sep+"part_"+part+".csv"

		if path.isfile(partfile):
			try:
				f=open(partfile,"r")
				r=f.readline()


				while r != "":
					r1=r.split(",")
					row_id=r1[0]
					ls=float(r1[1])
					bs=float(r1[2])
					stringa_s=r1[3].strip("\r\n")
	
					if fwhm1*fwhm2!=0:
						check,dist = matchVerify(ls, bs, glonc, glatc, fwhm1, fwhm2, PA, shape)
						if check==1:
							#res=res+str(ind)+","+str(row_id)+","+stringa_c+","+stringa_s+","+str(dist)+"\n"
							res=res+str(ind)+","+str(row_id)+","+str(dist)+"\n"
							
					r=f.readline()					

				f.close()
			except:
				print("Error opening splitted file: "+partfile)

	return res			
	

def execution(area_params, tmp_path, n_threads, maindir, pep, outdata):

	set_globvar(area_params, tmp_path, n_threads, maindir)
	#rows=params[0]

	rows=open(tmp_path+sep+"group"+str(pep),"r")
	parts_string="0-0-0"
	outmatch=open(tmp_path+sep+str(pep)+".csv","w")
	outmatch.write("col1,col2,col3,col4,col5\n")
	outmatch.close()	
	#print(catalogo)
	
	line=rows.readline()
	while line!="":
	#for line in rows:

		r=line.split(",")
		C_ID=r[0]

		glonc=float(r[3])
		glatc=float(r[4])
		PA=float(r[7])*clock+shift
		FWHMa=float(r[5])
		FWHMb=float(r[6])

		ind=int(r[8])
		stringa=r[9].strip("\n")

		

		if (glonc<=lmax) and (glonc>=lmin) and (glatc<=bmax) and (glatc>=bmin):

			#FWHM_ABmax=max(FWHMa,FWHMb)
			#FWHM_ABmin=min(FWHMa,FWHMb)
			FWHMmax=max(FWHMa,FWHMb)/3600
			parts_string=assignPart(glonc, glatc, FWHMmax, subdiv_l, n_parts_l, subdiv_b, n_parts_b, lmin, lmax, bmin, bmax)

		if parts_string!="0-0-0":
			#time.sleep(0.4)
			res=matchSky(C_ID,glonc,glatc,PA,FWHMa,FWHMb,parts_string,tmp_path,ind,shape, stringa)
			outmatch=open(tmp_path+sep+str(pep)+".csv","a")
			outmatch.write(res)
			outmatch.close()
		line=rows.readline()
	rows.close()

	if n_threads>10:
		if pep%10==0:
			sys.stdout.write(".")
			sys.stdout.flush()
	else:
		sys.stdout.write("...")
		sys.stdout.flush()
		




def loadAreaParams(params,tmp_path):

	with open(tmp_path+sep+"params","r") as f:
		first=f.readline().rstrip("\n")
		part1=first.split(",")
#		print(part1)

		second=f.readline().rstrip("\n")

		subdiv_l=[ast.literal_eval(second)]
#		subdiv_l=list(second)
#		print(subdiv_l)

		third=f.readline().rstrip("\n")
		subdiv_b=[ast.literal_eval(third)]
#		print(subdiv_b)


	area_params=part1+subdiv_l+subdiv_b+params

	return area_params

def skyRun(params,tmp_path, n_threads, maindir, outdata):

	warnings.filterwarnings("ignore")		#SUPPRESSED WARNING BY np.array_split() in case of void row_group
	parts_string="0-0-0"
	thread=[]

	area_params=loadAreaParams(params, tmp_path)
	
	
	jobs=[]

	for i in range(n_threads):
		idObj=i+1

		if path.isfile(tmp_path+sep+"group"+str(i)):
			proc=Process(target=execution, args=(area_params, tmp_path, n_threads, maindir,i,outdata,))
			proc.start()
			jobs.append(proc)

	for j in jobs:
		j.join()

