#!/usr/bin/env python
#!/usr/bin/python
#
# utility.py
# function library to provide utility to other functions
#
# Authors: G. Riccio
# Creation Date: January 20, 2016
# Release: 1.0
# Release Date: July 23, 2016
#
# Copyright 2016, Giuseppe Riccio, Massimo Brescia
# This file is part of C3.

    # C3 is free software: you can redistribute it and/or modify
    # it under the terms of the GNU General Public License as published by
    # the Free Software Foundation, either version 3 of the License, or
    # (at your option) any later version.

    # C3 is distributed in the hope that it will be useful,
    # but WITHOUT ANY WARRANTY; without even the implied warranty of
    # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    # GNU General Public License for more details.

    # You should have received a copy of the GNU General Public License
    # along with C3.  If not, see <http://www.gnu.org/licenses/>.
#
### WINDOWS INSTRUCTIONS
#
# This program has been produced with python 2.7.8 (64bit)
# It has been tested on Windows 7 and 8pro, by running it in a system
# shell window.
#
# IMPORTANT: be sure to have previously installed python 2.7.8
# 64-bit version together with the following external packages:
#
# 	numpy;
#
### LINUX UBUNTU INSTRUCTIONS
#
# This program has been tested on Linux Ubuntu 14.04 with python 2.7.6 (64-bit)
#
#################
# SYNOPSIS
#
# Simple python source code implementing utility functions 
#
###################
from 	os 			import sep, system, path, makedirs
from    math			import floor,ceil, degrees
import  csv
import  re
import 	numpy 	as np
import 	time
import  shutil
import 	sys


def getSelection():
	return np.array(["best", "all"])

def getJoinType():
	return np.array(["1 and 2", "1 or 2", "all from 1", "all from 2", "1 not 2", "2 not 1", "1 xor 2"])

def getCriteria():
	return np.array(["row-by-row", "sky", "exact value"])

def getMinFWHM():
	return 125			#DA OTTIMIZZARE

def getAreaShape():
	return np.array(["ellipse", "rectangle"])

def getErrorType():
	return np.array(["parametric", "fixed"])

def getCoordSys():
	return np.array(["icrs", "fk4", "fk5", "galactic"])

def getUnits():
	return np.array(["degrees","radians", "sexagesimal", "deg", "rad", "sex"])

def getFormats():
	return np.array(["csv","ascii", "fits", "votable"])

def setSeparator(filename):					# set the right separator for the OS in use
	outfile=filename.replace("\\",sep).replace("/",sep)
	return outfile

def getDistCase():
	return np.array(["1 and 2", "1 or 2", "all from 1", "all from 2"])  # Join type foreseing separation column

def findCol(header, colname,logfile, tmp_path):				   #finds the number of a column in a CSV file starting from its name
	col=0
	hdr=header.split(",")
	err=0

	if hdr[0]=="col1":
		try:
			col=int(colname)
			if col>len(hdr):
				print("\nError: Column index "+colname+" exceeds the size of the input file!Please check and retry.")
				writeLogFile(logfile, "Error: Column index "+colname+"exceeds the size of the input file", "a")
				err=1
	
		except:
			print("\nError: Column "+colname+" not found! Please check and retry.")
			writeLogFile(logfile, "Error: Column "+colname+"not found", "a")
			err=1

	else:
		try:
			col=int(colname)
			if col>len(hdr):
				print("\nError: Column index "+colname+" exceeds the size of the input file!Please check and retry.")
				writeLogFile(logfile, "Error: Column index "+colname+"exceeds the size of the input file", "a")
				err=1
		except:
			count=1
			col=0
			for r in hdr:
				if r==colname:
					col=count
				else:
					count=count+1			

	if err==1:
		errorExit(tmp_path)
		#exit(1)
	
	if col==0:
		print("\nError: Column "+colname+" not found! Please check and retry.")
		writeLogFile(logfile, "Error: Column "+colname+" not found\n", "a")
		errorExit(tmp_path)

	return col			


def findColMinMax(rows, col):	# finds minimum and maximum of a column in a CSV file
	
	l_array=[]
	for r in rows:
		line=r.replace("\n","").split(",")
		curr_value=line[col]
		if curr_value=="":			#if the value is null it is changed in 0 (for FWHMs)
			curr_value=0

		l_array.append(float(curr_value))
	
	l_min=min(l_array)
	l_max=max(l_array)

	return l_min, l_max


#def sortFile(filename, col, stiltsDir, tmp_path):	       # sorts a file according to the column "col" 
#	splitted_name=filename.replace("/",sep).replace("\\",sep)
#	splitted_name=split(splitted_name,sep)
#	sourcesName=str(splitted_name[-1:])
#	sourcesName=sourcesName.replace("['","").replace("']","")	
#	if sep=="/":
#		primo="'"
#		secondo='"'
#	else:
#		primo='"'
#		secondo="'"
#       cmd='java -jar '+stiltsDir+"stilts.jar tpipe cmd="+primo+"sort $"+str(col)+primo+" omode=out ifmt=csv ofmt=csv in="+filename+" out="+tmp_path+sep+"sorted_"+sourcesName
#	system(cmd)
#	return sourcesName


#def splitFile(filename, l, step, colname, tmp_path):	    # splits a file in shorter files
       
#	with open(filename, 'r') as csvfile:
#		reader = csv.DictReader(csvfile)
#		header=reader.fieldnames
#		area=1
#
#		#open outfile
#	       outputFile=tmp_path+sep+"part"+str(area)+".csv"
#		fout = open(outputFile,"wb")
#
#		writer=csv.DictWriter(fout, fieldnames=header, delimiter = ',')
#		writer.writeheader()
#
#		for row in reader:
#		    if (float(row[colname])<=l+area*step):
#			writer.writerow(row)
#		    else:
#			fout.close()
#			area=area+1
#			outputFile=tmp_path+sep+"part"+str(area)+".csv"
#			fout = open(outputFile,"wb")
#			writer=csv.DictWriter(fout, fieldnames=header, delimiter = ',')
#			writer.writeheader()
#			writer.writerow(row)
#		fout.close()


def assignPart(glon, glat, fwhm, subdiv_l, n_parts_l, subdiv_b, n_parts_b, lmin, lmax, bmin, bmax):		    # assigns to a clump the areas to analyze
	
	lc=glon
	bc=glat
	limdx=lc+fwhm/2
	limsx=lc-fwhm/2
	limsup=bc+fwhm/2
	liminf=bc-fwhm/2

	area_l=1
	area_b=1
	areasx=0
	areadx=0
	areasup=0
	areainf=0
	parts=""

	# find areas in longitude
	#central cell
	while lc>subdiv_l[area_l]:
		area_l=area_l+1


	# ELIF DI AREADX E AREASX SOLO SE LIMITI SONO 0,360


	#right cell
	if area_l<n_parts_l:
		if limdx>subdiv_l[area_l]:
			areadx=area_l+1
	elif area_l==int(n_parts_l) or (area_l==int(n_parts_l)-1 and subdiv_l[int(n_parts_l)]>360):
		areadx=1

	#left cell
	if area_l>1:
		if limsx<subdiv_l[area_l-1]:
			areasx=area_l-1
	elif area_l==1:
		areasx=int(n_parts_l)

	# find areas in latitude
	#central cell
	while bc>subdiv_b[area_b]:
		area_b=area_b+1

	#upper cell
	if area_b<n_parts_b:
		if limsup>subdiv_b[area_b]:
			areasup=area_b+1
	elif area_b==int(n_parts_b) or (area_b==int(n_parts_b)-1 and subdiv_b[int(n_parts_b)]>90):
		areasup=1

	#lower cell
	if area_b>1:
		if liminf<subdiv_b[area_b-1]:
			areainf=area_b-1
	elif area_b==1:
		areainf=int(n_parts_b)


	areas_lon=[area_l, areasx, areadx]
	areas_lat=[area_b, areasup, areainf]

	for i in areas_lon:
		for j in areas_lat:
			if i!=0 and j!=0:
				parts=parts+str(i)+"_"+str(j)+","
	
	parts=parts[:-1]
	return parts

		
#def selCol(mat, n_col):					 # extracts the n_col column from the array mat
#	i=0
#	mat_desig=[]
#	while i<len(mat):
#		parts=split(mat[i],",")
#		mat_desig.append(parts[n_col])
#		i=i+1
#	return mat_desig


#def findNringMax(rows, colA, colB):
#	
#	min_array=[]
#
#	for r in rows:
#		line=r.replace("\n","").split(",")
#		tempA=float(line[colA])
#		tempB=float(line[colB])
#		if tempA<=tempB:
#			min_array.append(tempA)
#		else:
#			min_array.append(tempB)
#	
#	return max(min_array)


def file2csv(nfile, nformat, stiltsDir, tmp_path, logfile):

	name_arr=nfile.split(sep)
	name=name_arr[-1]
	new_filename=tmp_path+sep+name+".csv"
	cmd="java -jar "+stiltsDir+"stilts.jar tcopy "+nfile+" "+new_filename+" ifmt="+nformat+" ofmt=csv"
	a=system(cmd)
	if a!=0:
		writeLogFile(logfile, "Error opening input file "+nfile+"\n", "a")		
		errorExit(tmp_path)
#		exit(1)
	return new_filename


def writeLogFile(logfile, line, mode):
	filelog=open(logfile,mode)
	filelog.write(line)
	filelog.close()



#def findLimits(glon2_array, glat2_array):
#	lmin=min(glon2_array)
#	lmax=max(glon2_array)
#	bmin=min(glat2_array)
#	bmax=max(glat2_array)
#	dl=lmax-lmin
#	db=bmax-bmin
#
#	return lmin, lmax, bmin, bmax, dl, db


def correctedDistance(a,b, coord):
	
	# a & b are in arcsecs

	if coord=="glon":
		shift=360*3600
	elif coord=="glat":
		shift=180*3600

	#print(str(a)+","+str(b))

	#dist1=abs(a-b)
	#dist2=abs(a-b-shift)
	#dist3=abs(a-b+shift)
	#return min(dist1, dist2, dist3)

	dist1=a-b
	dist2=a-b-shift
	dist3=a-b+shift

	if abs(dist3)<abs(dist2) and abs(dist3)<abs(dist1):
		return dist3
	elif abs(dist2)<abs(dist1) and abs(dist2)<abs(dist3):
		return dist2
	else:
		return dist1
		






def writeTime(t, logfile):
	c=time.strftime("%H:%M:%S",time.localtime(t))
	writeLogFile(logfile, c+" :\t", "a")

def createConfig(filename):

	if path.isfile(filename):
		print("\nError! Configuration file "+filename+" already exists.\n")
		exit(1)



	config='''[I/O Files]

Input catalogue 1:	./file1.csv
Format catalogue 1:	csv				;csv, fits, votable or ascii

Input catalogue 2:	./file2.csv
Format catalogue 2:	csv				;csv, fits, votable or ascii

Output:			./output/output.csv
Output format:		csv				;csv, fits, votable or ascii
Log file:		./output/out.log
Stilts directory:	./libs
working directory:	./tmp				;temporary directory, removed after completed

[Match Criteria]
algorithm:		sky				;sky, exact value, row-by-row

[Exact Value parameters]
;required for algorithm: "exact value"			
Input 1 column:		ID				;match column catalog 1
Input 2 column:		1				;match column catalog 2

[Row by Row parameters]
;no specific parameters are required for algorithm: "row-by-row"			

[Sky parameters]
;required for algorithm: "sky"
area shape:		ellipse				;ellipse or rectangle
size type:		fixed				;parametric or fixed
matching area first dimension:		10		;arcsec in case of fixed type - column name/number in case of parametric type 
matching area second dimension:		10		;arcsec in case of fixed type - column name/number in case of parametric type 
parametric factor:			1		;multiplicative factor for dimension columns - required for parametric type
pa column/value:	45				;degrees in case of fixed type - column name/number in case of parametric type -missing or default = 0 deg
pa settings:		clock, 0			;orientation (clock, counter), shift (degrees) - missing or default = clock,0)
Catalogue 2 minimum partition cell size:125		;arcsec, (optional, default=100 arcsecs)

[Catalogue 1 Properties]
coordinate system:	galactic			;galactic, icrs, fk4, fk5
coordinate units:	deg				;degrees (or deg), radians (or rad), sexagesimal (or sex)
glon/ra column:		GLON				;column number or name - required for error type fwhm
glat/dec column:	GLAT				;column number or name - required for error type fwhm
designation column:	ID1				;column number or name - -1 for none


[Catalogue 2 Properties]
coordinate system:	icrs				;galactic, icrs, fk4, fk5
coordinate units:	deg				;degrees (or deg), radians (or rad), sexagesimal (or sex)
glon/ra column:		RA				;column number or name
glat/dec column:	DEC				;column number or name
designation column:	ID2				;column number or name - -1 for none - available only for area shape ellipse


[Threads Properties]
thread limit: 		4				;maximum number of simultaneous threads (it depends on your machine)

[Output Rows]
Match selection:	all				;all or best
Join type:		1 and 2				;1 and 2, 1 or 2, all from 1, all from 2, 1 not 2, 2 not 1, 1 xor 2'''


	f=open(filename,"w")
	f.write(config)
	f.close()



def errorExit(tmp):
	shutil.rmtree(tmp)
	exit(1)



def removeTmp(logfile,tmp_path):

	### Removing temporary files
	sys.stdout.write("Removing temporary files........")
	sys.stdout.flush()
	t3=time.time()
	writeTime(t3, logfile)
	writeLogFile(logfile, "Removing tmp phase started\n", "a")
	shutil.rmtree(tmp_path)
	t_rem=time.time()-t3
	print(" DONE (duration: %4.2fs)\n" % (t_rem))
	writeLogFile(logfile, "\t\tRemoving tmp phase duration: %4.2fs" % (t_rem)+"\n\n", "a")

